import java.io.*;
import java.util.*;
import java.util.stream.*;
import java.text.*;

import au.com.fax2.*;
import org.apache.http.client.ClientProtocolException;

public class ReceiveFaxes {
	static final int MARGIN = 300;

	static SimpleDateFormat dateFormatter;
	
	static {
		dateFormatter = new SimpleDateFormat("yyyyMMddHHmm");
		dateFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
	}

	private static final String propertiesFilename = "fax2demo.properties";
	
	Properties properties;
	Client client;
	Set<String> lastSeen;
	Date from_time;
	
	public ReceiveFaxes()
		throws IOException
	{
		properties = new Properties();
		FileInputStream propertiesFile = new FileInputStream(propertiesFilename);
		properties.load(propertiesFile);
		propertiesFile.close();

		client = new Client(properties);
		from_time = new Date();
		lastSeen = new HashSet<String>();
	}
	
	public void runOnce()
		throws ClientProtocolException,IOException,Client.Exception,
			InterruptedException
	{
		Client.ReceivedFaxCollection list = client.getReceivedFaxes(from_time);
		if(!list.isEmpty()) {
			for (Client.ReceivedFax fax: list) {
				if(!lastSeen.contains(fax.id)) {
					String filename = "received-" +
						dateFormatter.format(fax.received_at) +
						"-" + fax.id + ".pdf";
					File file = new File(filename);
					if(file.exists()) {
						System.out.printf("%s already exists, skipping\n",filename);
					} else {
						String tmpname = String.format(
							"%s.tmp-%d",filename,(int)(Math.random() * 1e9)
						);
						client.writeReceivedFax(fax.id,tmpname);
						new File(tmpname).renameTo(file);
						System.out.printf("received fax %s written as %s\n",fax.id,filename);
					}
				}
				lastSeen.add(fax.id);
			}
			lastSeen.clear();
			for (Client.ReceivedFax fax: list) lastSeen.add(fax.id);
			
			long newFromMsec = list.stream().map(fax -> fax.received_at).
				max((a,b) -> a.compareTo(b)).
				get().getTime() - MARGIN * 1000;
			if(newFromMsec > from_time.getTime())
				from_time = new Date(newFromMsec);
		}
	}
	
	public void run() throws InterruptedException {
		while(true) {
			try {
				runOnce();
			} catch (Throwable e) {
				System.err.printf("Caught exception:\n");
				e.printStackTrace(System.err);
			}
			Thread.sleep(1000 * 30);
		}
	}

	public static void main(String[] args) throws java.lang.Exception {
		new ReceiveFaxes().run();
	}
}
